/*
File: AppletHandle.java
Date		Author		Changes		
Sept 09 96	Lukasz Beca	Created
*/

package webwisdom.tango.structures;

import webwisdom.tango.structures.Handle;
import webwisdom.tango.interfaces.AppletBaseInterf;
import webwisdom.tango.messages.AppEventMessage;
import webwisdom.tango.messages.EmptyMessage;
import webwisdom.tango.messages.Message;
import webwisdom.tango.messages.EventMessage;
import webwisdom.tango.consts.Const;
import netscape.javascript.JSObject;
import webwisdom.tango.structures.LocalApplications;


public class AppletHandle implements Handle, Runnable {

  private AppletBaseInterf app;
  private int AID;
  private LocalApplications lapps;
  private MessageQueue q;
  private boolean exit; 
  private Thread th;
  private int closeMode;
 
  public AppletHandle(AppletBaseInterf a, int AIDpar, LocalApplications localpar) {

    app = a;
    AID = AIDpar;
    lapps = localpar;
    q = new MessageQueue();
    th = new Thread(this);
    exit = false;
    closeMode = Const.NONE;
    th.start();
    
  }

  //interface for LocalApplications
  //public void sendLAToApp(Message m);
  //public synchronized void finish();
  //public synchronized void setFocus();
 
  public void sendLAToApp(Message m) {
    
    q.putMessage(m);

  }
  
  public synchronized void finish() {

    if (closeMode == Const.NONE) {
    
      System.out.println("AppletHandle: finishing application");
      closeMode = Const.CACLOSED;
      caTermination();

    }
  }


  public synchronized void setFocus() {

   JSObject win, parent;

   win = app.getJSWindow();
   parent = (JSObject) win.getMember("top");
   if (parent == null) {
      System.out.println("AppletHandle: unable to get reference to parent");
      return;
   }
   parent.eval("focus();");
  }
  

  //interface for AppletBase
  //public void sendAppToOther(AppEventMessage m);
  //public void sendAppToOther(AppEventMessage m, int userlist[]);
  //public synchronized void remove();

  public void sendAppToOther(AppEventMessage m) {

   lapps.sendAppToOther(AID, m);

  }
  
  public void sendAppToOther(int userlist[], AppEventMessage m) {
  
  	lapps.sendAppToOther(AID, userlist, m);
  
  }
  public synchronized void remove() {

    if (closeMode == Const.NONE) {

      closeMode = Const.WINDOWCLOSED;
      System.out.println("AppletHandle: removing applet " + AID);
      lapps.terminated(AID);
      exit = true;
      q.unblock();    
    }
  }


  //interface for Thread
  //public void run();

  public void run() {

    Message m;
    AppEventMessage am;
   
    System.out.println("AppletHandle: applet thread number " + AID+ "  started");
   
    
    while(!exit) {

      //System.out.println("ThreadOut: Thread number " + thisno + " sending");
	m = q.getMessage();	

	synchronized (this) {
	  if (!exit) {
	    am = (AppEventMessage) m;
	    app.receive(am);
	  }
	}    
    }
    System.out.println("AppletHandle: message queue released");
   
    shutdown();
   
  }

  //internal methods
  //private void caTermination();
  //private void windowTermination();
  //private void shutdown();

  private void caTermination() {

    JSObject win, parent;
   

    win = app.getJSWindow();

    if (win == null) {
      System.out.println("AppletHandle: unable to get reference to window");
      return;
    }

    parent = (JSObject) win.getMember("top");

    if (parent == null) {
      System.out.println("AppletHandle: unable to get reference to parent");
      return;
    }

    System.out.println("AppletHandle: terminating applet " + AID);

    if (app.inWindow()) {
     
      parent.eval("close();");
    }
    System.out.println("AppletHandle: window closed");
    
    
  }
  private void windowTermination(){

   
  }

  private void shutdown() {
    
    q.clear();
    q = null;
    app = null;
    lapps = null;
    th = null;

    System.out.println("AppletHandle: associations released");
  }

  
}












